package main;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JSlider;
import javax.swing.UIManager;
import javax.swing.UIManager.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import model.*;
import parser.*;
import utilities.DataDensity;
import utilities.DataType;
import utilities.Dimensionality;
import utilities.Goal;
import utilities.Interaction;
import utilities.StaticVariables;
import utilities.User;
import view.*;

public class MainProgramImpl implements MainProgram {

	private final MainAlgorithm algorithm = new MainAlgorithmImpl();
	private final XMLcreator fileParser = new XMLcreatorImpl();
	private final QueryManager queryManager = new QueryManagerImpl();
	private final JFileChooser fileChooser = new JFileChooser();
	private final GUI gui;

	final Map<String, Integer> fileResult = new LinkedHashMap<>();
	final List<Map<String, Float>> queryResult = new ArrayList<>();
	final Map<String, Float> algorithmResult = new LinkedHashMap<>();

	final List<String> valuesList = new ArrayList<>();
	final List<Integer> prioritiesList = new ArrayList<>();

	public MainProgramImpl() {	
		
		try {
		    for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
		        if ("Nimbus".equals(info.getName())) {
		            UIManager.setLookAndFeel(info.getClassName());
		            break;
		        }
		    }
		} catch (Exception e) {
		    e.printStackTrace();
		}
		
		gui  = new GUIimpl2();
		
	}

	@Override
	public void run() {
		addActionListener();
	}

	private void addActionListener() {

		for (JComboBox<String> box : gui.getCoordinatesInput()) {
			box.addActionListener(new ComboBoxActionListener());
		}

		for (JSlider slider : gui.getPrioritiesInput()) {
			slider.addChangeListener(new SliderListener());
		}

		for (JSlider slider : gui.getFGinput()) {
			slider.addChangeListener(new SliderListener());
		}
		
		for(JCheckBox box : gui.getCheckBoxes()) {
			box.addActionListener(new CheckBoxListener());
		}

		gui.getImportButton().addActionListener(new RunActionListener());

	}
	
	class CheckBoxListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			
			MainProgramImpl.this.valuesList.clear();
			MainProgramImpl.this.prioritiesList.clear();

			/* Write XML file */
			for (Map.Entry<String, Integer> entry : MainProgramImpl.this.gui.getSelectedItems().entrySet()) {
				MainProgramImpl.this.valuesList.add(entry.getKey());
				MainProgramImpl.this.prioritiesList.add(entry.getValue());
			}

			fileParser.createXML(valuesList, prioritiesList);

			MainProgramImpl.this.runAlgorithm(StaticVariables.XML_PATH);
			
		}
	}
	
	class SliderListener implements ChangeListener {
		public void stateChanged(ChangeEvent e) {
			
			MainProgramImpl.this.valuesList.clear();
			MainProgramImpl.this.prioritiesList.clear();

			/* Write XML file */
			for (Map.Entry<String, Integer> entry : MainProgramImpl.this.gui.getSelectedItems().entrySet()) {
				MainProgramImpl.this.valuesList.add(entry.getKey());
				MainProgramImpl.this.prioritiesList.add(entry.getValue());
			}

			fileParser.createXML(valuesList, prioritiesList);

			MainProgramImpl.this.runAlgorithm(StaticVariables.XML_PATH);
			
		}
	}

	class ComboBoxActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {

			MainProgramImpl.this.valuesList.clear();
			MainProgramImpl.this.prioritiesList.clear();

			/* Write XML file */
			for (Map.Entry<String, Integer> entry : MainProgramImpl.this.gui.getSelectedItems().entrySet()) {
				MainProgramImpl.this.valuesList.add(entry.getKey());
				MainProgramImpl.this.prioritiesList.add(entry.getValue());
			}

			fileParser.createXML(valuesList, prioritiesList);

			MainProgramImpl.this.runAlgorithm(StaticVariables.XML_PATH);
		}
	}

	/* ActionListener of button, which reads an imported XML file */
	class RunActionListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			
			int returnVal = fileChooser.showOpenDialog(null);

	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	            File file = fileChooser.getSelectedFile();
	            
	            MainProgramImpl.this.runAlgorithm(file.getPath());            
	        } 
		}
	}

	private void runAlgorithm(String path) {

		final XMLreader fileReader = new XMLreaderImpl();

		/* Clears previous result */
		fileResult.clear();
		queryResult.clear();
		algorithmResult.clear();

		/* Read XML file */
		fileResult.putAll(fileReader.readXML(path));

		/* Fire queries */
		for (Map.Entry<String, Integer> entry : fileResult.entrySet()) {
			int coordinate = -1;
			String coordinateValue = entry.getKey();

			/* Determines the coordinate type of each entry */
			if (Goal.getAllValuesAsList().contains(coordinateValue)) {
				coordinate = StaticVariables.GOAL;
			} else if (DataDensity.getAllValuesAsList().contains(coordinateValue)) {
				coordinate = StaticVariables.DATA_DENSITY;
			} else if (Interaction.getAllValuesAsList().contains(coordinateValue)) {
				coordinate = StaticVariables.INTERACTION;
			} else if (Dimensionality.getAllValuesAsList().contains(coordinateValue)) {
				coordinate = StaticVariables.DIMENSIONALITY;
			} else if (DataType.getAllValuesAsList().contains(coordinateValue)) {
				coordinate = StaticVariables.DATA_TYPE;
			} else if (User.getAllValuesAsList().contains(coordinateValue)) {
				coordinate = StaticVariables.USER;
			} else {
			}

			if(coordinate != -1) {
				queryResult.add(queryManager.SearchDB(coordinate, coordinateValue));
			}
		}

		/* Runs algorithm */
		algorithmResult.putAll(algorithm.getResults(queryResult, fileResult));

		/* Displays results */
		gui.visualizeResults(algorithmResult);

	}

}