package model;

import java.util.List;
import java.util.Map;

/**
 * The main algorithm which returns the best graphs to visualize data according
 * to the user's preferences.
 * 
 * How the algorithm works:
 * 
 * 1. The SQL query made on the database returns the list of graphs which have
 * at least one of the coordinate values submitted by the user. For example, if
 * the input is "Goal = Composition, DataDensity = Low", the query returns each
 * graph that have Composition in the goal table, and each graph which have Low
 * in the datadensity table. Duplicates are to be expected. For each graph, the
 * query also returns a value that indicates the "strength" of the graph for the
 * chosen value. 2. The algorithm makes a separate graph->strength Map for each
 * coordinate chosen by the user. 3. For each graph, multiplies the strength for
 * the priority chosen by the user for that particular coordinate type. 4. For
 * each graph, adds the strength of each coordinate possessed by it. 5. Shows
 * the results, oredered by strength (DESC).
 * 
 * @author michael.giambi
 *
 */

public interface MainAlgorithm {

	/**
	 * 
	 * @param inputList
	 *            The query result.
	 * @param userPriorities
	 *            The list of priorities set by the user.
	 * @return For each graph chosen by the algorithm, its name and strength.
	 */

	public Map<String, Float> getResults(final List<Map<String, Float>> inputList,
			final Map<String, Integer> userPriorities);

}
