package model;

import java.util.*;
import java.util.Map.Entry;

import utilities.*;

public class MainAlgorithmImpl implements MainAlgorithm {

	public MainAlgorithmImpl() {
	}

	@Override
	public Map<String, Float> getResults(final List<Map<String, Float>> inputList,
			final Map<String, Integer> userPriorities) {

		/* Initialize variables */
		final List<String> graphList = this.createGraphList(inputList);
		final Float[] graphOccurrencies = new Float[graphList.size()];

		for (int i = 0; i < graphOccurrencies.length; i++) {
			graphOccurrencies[i] = 0f;
		}

		final Map<String, Float> result = new LinkedHashMap<>();
		final List<String> coordinateValues = new ArrayList<>();
		String coordinate = "";
		String tempGraph;
		Float tempOccurrency = 0f;
		int index = 0;
		int coordinatePriority = 0;
		int fiPriority = 1;

		/* Calculates the strength of each graph */
		for (int i = 0; i < graphList.size(); i++) {
			for (Map<String, Float> m : inputList) {
				for (Entry<String, Float> entry : m.entrySet()) {
					
					/*
					 * If the entry is the first entry of the map, which
					 * contains the coordinate type...
					 */
					if (entry.getValue() == StaticVariables.COORDINATE_INDEX) {

						coordinateValues.clear();

						switch (Integer.parseInt(entry.getKey())) {

						case StaticVariables.GOAL:
							coordinateValues.addAll(Goal.getAllValuesAsList());
							coordinate = StaticVariables.COORDINATES_NAMES[StaticVariables.GOAL];
							break;
						case StaticVariables.DATA_DENSITY:
							coordinateValues.addAll(DataDensity.getAllValuesAsList());
							coordinate = StaticVariables.COORDINATES_NAMES[StaticVariables.DATA_DENSITY];
							break;
						case StaticVariables.DATA_TYPE:
							coordinateValues.addAll(DataType.getAllValuesAsList());
							coordinate = StaticVariables.COORDINATES_NAMES[StaticVariables.DATA_TYPE];
							break;
						case StaticVariables.DIMENSIONALITY:
							coordinateValues.addAll(Dimensionality.getAllValuesAsList());
							coordinate = StaticVariables.COORDINATES_NAMES[StaticVariables.DIMENSIONALITY];
							break;
						case StaticVariables.INTERACTION:
							coordinateValues.addAll(Interaction.getAllValuesAsList());
							coordinate = StaticVariables.COORDINATES_NAMES[StaticVariables.INTERACTION];
							break;
						case StaticVariables.USER:
							coordinateValues.addAll(User.getAllValuesAsList());
							coordinate = StaticVariables.COORDINATES_NAMES[StaticVariables.USER];
							break;
						default:
							break;

						}

						for (Map.Entry<String, Integer> entry2 : userPriorities.entrySet()) {
							if (coordinateValues.contains(entry2.getKey())) {
								coordinatePriority = entry2.getValue();
							}

							if (entry2.getKey().equals(StaticVariables.FI_NAMES[StaticVariables.OPERATIVITY])) {
								if (StaticVariables.OPERATIVITY_NAMES.contains(coordinate)) {
									fiPriority = entry2.getValue();
								}
							}

							if (entry2.getKey().equals(StaticVariables.FI_NAMES[StaticVariables.DATA_MODELING])) {
								if (StaticVariables.DATA_MODELING_NAMES.contains(coordinate)) {
									fiPriority = entry2.getValue();
								}
							}
						}
					}

					if (graphList.get(i).equals(entry.getKey())) {
						if (entry.getValue() == -1) {
							graphOccurrencies[i] += -9999;
						} else {
							graphOccurrencies[i] = graphOccurrencies[i] + (entry.getValue() * coordinatePriority * fiPriority);
						}
					}
				}
			}
		}

		final List<Float> occurrencies = new LinkedList<Float>(Arrays.asList(graphOccurrencies));
		int bound = graphList.size();

		if (bound > StaticVariables.nResults) {
			bound = StaticVariables.nResults;
		}

		/* Returns the X graphs with the higher strength */
		for (int j = 0; j < bound; j++) {

			tempGraph = graphList.get(0);
			tempOccurrency = 0f;
			index = 0;

			for (int i = 0; i < graphList.size(); i++) {
				if (occurrencies.get(i) > tempOccurrency) {
					tempOccurrency = occurrencies.get(i);
					tempGraph = graphList.get(i);
					index = i;
				}
			}

			result.put(tempGraph, tempOccurrency);

			graphList.remove(index);
			occurrencies.remove(index);
		}

		return result;
	}

	/* Puts all the graphs retrieved by the query in a single list */
	private List<String> createGraphList(final List<Map<String, Float>> inputList) {

		final List<String> result = new ArrayList<>();

		for (Map<String, Float> m : inputList) {
			for (Map.Entry<String, Float> entry : m.entrySet()) {

				if (!result.contains(entry.getKey()) && entry.getValue() != StaticVariables.COORDINATE_INDEX) {
					result.add(entry.getKey());
				}
			}
		}

		return result;
	}

}
