package model;

import java.sql.*;
import java.util.*;
import utilities.*;

public class QueryManagerImpl implements QueryManager {

	@Override
	public Map<String, Float> SearchDB(final Integer coordinate, final String value) {

		String table = "";

		switch (coordinate) {
		case StaticVariables.GOAL:
			table = "goal";
			break;
		case StaticVariables.DATA_DENSITY:
			table = "datadensity";
			break;
		case StaticVariables.DATA_TYPE:
			table = "datatype";
			break;
		case StaticVariables.DIMENSIONALITY:
			table = "dimensionality";
			break;
		case StaticVariables.INTERACTION:
			table = "interaction";
			break;
		case StaticVariables.USER:
			table = "user";
			break;
		default:
			break;
		}

		final Map<String, Float> results = new LinkedHashMap<>();
		final Connection connection = this.connect();

		/*
		 * The first element of the map contains the coordinate type to which
		 * the query refers
		 */
		results.put(coordinate.toString(), (float)StaticVariables.COORDINATE_INDEX);

		/*
		 * returns the graph name and score of each graph with the selected
		 * coordinate value
		 */
		final String query = "SELECT subcategory, category, score " + "FROM " + table + " INNER JOIN graphs " + "ON "
				+ table + ".graph = graphs.idgraphs " + "WHERE value = '" + value + "'";

		try {
			final Statement statement = connection.createStatement();
			final ResultSet res = statement.executeQuery(query);

			while (res.next()) {
				results.put(res.getString("subcategory") + "-" + res.getString("category"), res.getFloat("score"));
			}

			connection.close();
		} catch (Exception e) {
			e.printStackTrace();
		}

		return results;
	}

	@Override
	public void writeHistory(Map<String, Integer> input) {
		
		List<String> coordinatesList = new ArrayList<>();
		List<Integer> prioritiesList = new ArrayList<>();
		
		for(Map.Entry<String, Integer> entry : input.entrySet()) {
			coordinatesList.add(entry.getKey());
			prioritiesList.add(entry.getValue());
		}
		
		try
	    {
	      // create a mysql database connection
	      String myDriver = "org.gjt.mm.mysql.Driver";
	      String myUrl = "jdbc:mysql://localhost/test";
	      Class.forName(myDriver);
	      Connection conn = DriverManager.getConnection(myUrl, "root", "");
	    
	      // create a sql date object so we can use it in our INSERT statement
	      Calendar calendar = Calendar.getInstance();
	      java.sql.Date startDate = new java.sql.Date(calendar.getTime().getTime());

	      // the mysql insert statement
	      String query = " insert into history (idhistory, date, goal, dimensionality, dataDensity, interaction, dataType, user, goal_priority, dimensionality_priority, dataDensity_priority, interaction_priority, dataType_priority, user_priority)"
	        + " values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

	      // create the mysql insert preparedstatement
	      PreparedStatement preparedStmt = conn.prepareStatement(query);
	      preparedStmt.setInt (1, 0);
	      preparedStmt.setDate (2, startDate);
	      preparedStmt.setString (3, coordinatesList.get(StaticVariables.GOAL));
	      preparedStmt.setString (4, coordinatesList.get(StaticVariables.DIMENSIONALITY));
	      preparedStmt.setString (5, coordinatesList.get(StaticVariables.DATA_DENSITY));
	      preparedStmt.setString (6, coordinatesList.get(StaticVariables.INTERACTION));
	      preparedStmt.setString (7, coordinatesList.get(StaticVariables.DATA_TYPE));
	      preparedStmt.setString (8, coordinatesList.get(StaticVariables.USER));
	      preparedStmt.setInt (9, prioritiesList.get(StaticVariables.GOAL));
	      preparedStmt.setInt (10, prioritiesList.get(StaticVariables.DIMENSIONALITY));
	      preparedStmt.setInt (11, prioritiesList.get(StaticVariables.DATA_DENSITY));
	      preparedStmt.setInt (12, prioritiesList.get(StaticVariables.INTERACTION));
	      preparedStmt.setInt (13, prioritiesList.get(StaticVariables.DATA_TYPE));
	      preparedStmt.setInt (14, prioritiesList.get(StaticVariables.USER));

	      // execute the preparedstatement
	      preparedStmt.execute();
	      
	      conn.close();
	    }
	    catch (Exception e)
	    {
	      e.printStackTrace();
	    }

	}

	private Connection connect() {

		Connection connection = null;

		try {
			Class.forName("com.mysql.jdbc.Driver");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}

		try {
			connection = DriverManager.getConnection(StaticVariables.mySQLpath, StaticVariables.username,
					StaticVariables.password);
		} catch (SQLException e) {
			e.printStackTrace();
		}

		if (connection == null) {
			System.out.println("ERROR: CONNECTION FAILED");
		}

		return connection;
	}

}
