package view;

import java.util.Map;
import javax.swing.*;
import java.util.*;

/**
 * The graphical user interface of the program, for testing purpose only.
 * 
 * @author michael.giambi
 *
 */

public interface GUI {

	/**
	 * Gets the list of coordinate values chosen by the user
	 * 
	 * @return The coordinate comboboxes.
	 */
	public List<JComboBox<String>> getCoordinatesInput();

	/**
	 * Gets the list of priorities chosen by the user
	 * 
	 * @return The priority comboboxes.
	 */
	public List<JSlider> getPrioritiesInput();

	/**
	 * Gets the list of FG priorities chosen by the user.
	 * 
	 * @return The list of FG priorities chosen by the user.
	 */
	public List<JSlider> getFGinput();

	public List<JCheckBox> getCheckBoxes();

	/**
	 * Gets the Button which runs the algorithm with an already exsisting XML
	 * file.
	 * 
	 * @return The run Button
	 */
	public JButton getImportButton();

	/**
	 * Gets the list of coordinates chosen by the user.
	 * 
	 * @return The list of coordinates chosen by the user.
	 */
	public Map<String, Integer> getSelectedItems();

	/**
	 * Displays the results of the main algorithm.
	 * 
	 * @param results
	 *            the results of the algorithm.
	 */
	public void visualizeResults(final Map<String, Float> results);

}
